package article_test

import (
	"encoding/json"
	"testing"

	article "gihyo.jp/when-in-go/30-Code-Generation"
)

func TestJSON(t *testing.T) {
	testcases := []struct {
		Name   string
		JSON   string
		Object interface{}
		Error  bool
	}{
		{
			Name:   `Object1: Valid JSON`,
			Object: &article.Object1{},
			JSON:   `{"foo":"one","bar":2,"baz":true}`,
		},
		{
			Name:   `Object1: Invalid JSON`,
			Object: &article.Object1{},
			JSON:   `{"foooooooooooo":"one","bar":2,"baz":true}`,
			Error:  true,
		},
		{
			Name:   `Object2: Valid JSON`,
			Object: &article.Object2{},
			JSON:   `{"hoge":"one","fuga":2,"mogo":true}`,
		},
		{
			Name:   `Object2: Invalid JSON`,
			Object: &article.Object2{},
			JSON:   `{"hoge":"one","fuuuuuuuuuuuuuuuuga":2,"mogo":true}`,
			Error:  true,
		},
	}

	for _, tc := range testcases {
		tc := tc
		t.Run(tc.Name, func(t *testing.T) {
			err := json.Unmarshal([]byte(tc.JSON), tc.Object)
			if tc.Error {
				if err == nil {
					t.Fatal(`expected error, got none`)
				}
			} else {
				if err != nil {
					t.Fatalf(`expected success, got %s`, err)
				}
			}
		})
	}
}
